@@ -119,6 +119,17 @@ sub _cmd_map {
     return $cmd;
 }
 
+# rebless to subcommand class if eixsts
+sub _subcommand {
+    my ($self) = @_;
+    no strict 'refs';
+    for (grep {$self->{$_}} values %{{$self->options}}) {
+	if (exists ${ref($self).'::'}{$_.'::'}) {
+	    return bless ($self, (ref($self)."::$_"));
+	}
+    }
+}
+
 =head3 invoke ($xd, $cmd, $output_fh, @args)
 
 Takes a L<SVK::XD> object, the command name, the output scalar reference,
@@ -146,6 +157,7 @@ sub invoke {
 	$cmd = get_cmd ($pkg, $cmd, $xd);
 	$cmd->{svnconfig} = $xd->{svnconfig} if $xd;
 	$cmd->getopt (\@args, 'h|help|?' => \$help);
+	$cmd->_subcommand;
 
 	# Fake shell globbing on Win32 if we are called from main
 	if (IS_WIN32 and caller(1) eq 'main') {
@@ -162,11 +174,17 @@ sub invoke {
 	}
 	else {
 	    $cmd->msg_handler ($SVN::Error::FS_NO_SUCH_REVISION);
-	    eval { $cmd->lock (@args); $ret = $cmd->run (@args) };
+	    eval { $cmd->lock (@args);
+		   $xd->giant_unlock if $xd && !$cmd->{hold_giant};
+		   $ret = $cmd->run (@args) };
 	    $xd->unlock if $xd;
 	    die $@ if $@;
 	}
     };
+
+    # in case parse_arg dies
+    $xd->giant_unlock if $xd && ref ($cmd) && !$cmd->{hold_giant};
+
     $ofh = select STDERR unless $output;
     unless ($error and $cmd->handle_error ($error)) {
 	print $ret if $ret;
@@ -264,7 +282,7 @@ sub run {
 
 =head2 Utility Methods
 
-Except for C<arg_depotname>, all C<args_*> methods below returns a
+Except for C<arg_depotname>, all C<arg_*> methods below returns a
 L<SVK::Target> object, which consists of a hash with the following keys:
 
 =over
@@ -396,10 +414,23 @@ sub arg_uri_maybe {
         last;
     }
 
+    my $prompt = loc("
+First, you need to choose a depot path to store this mirrored repository,
+usually something like //mirror/your_project/.  Enter a suitable name below
+to place it under //mirror/; you may also press enter and accept the default.
+
+");
+
+    my $default = $base_uri->path;
+    $default =~ s{^/+|/+$}{}g;
+    $default =~ s{(?:/(?=trunk$)|/(?:tags|branche?s)/(?=[^/]+$))}{-};
+    $default =~ s{.*/}{};
+
     my $path = get_prompt(
-        loc("Name a depot path for this mirror (under //mirror/ if no leading '/'): "),
-        qr{^(?:/(?:$depots)/)?[^/]},
+        $prompt . loc("Depot path: [//mirror/%1] ", $default),
+        qr{^(?:$|(?:/(?:$depots)/)?[^/])},
     );
+    $path = $default unless length $path;
     $path = "//mirror/$path" unless $path =~ m!^/!;
 
     my $target = $self->arg_depotpath($path);
@@ -434,7 +465,8 @@ sub arg_uri_maybe {
 
 =head3 arg_co_maybe ($arg)
 
-Argument might be a checkout path or a depotpath.
+Argument might be a checkout path or a depotpath. If argument is URI then
+handles it via C<arg_uri_maybe>.
 
 =cut
 
@@ -590,19 +622,11 @@ XXX Undocumented
 =cut
 
 sub lock_target {
-    my ($self, $target) = @_;
-    $self->{xd}->lock ($target->{copath});
-}
-
-=head3 lock_none ()
-
-XXX Undocumented
-
-=cut
-
-sub lock_none {
-    my ($self) = @_;
-    $self->{xd}->giant_unlock ();
+    my $self = shift;
+    for my $target (@_) {
+	$self->{xd}->lock ($target->{copath})
+	    if $target->{copath};
+    }
 }
 
 =head3 brief_usage ($file)
@@ -729,7 +753,7 @@ sub command {
     my ($self, $command, $args, $is_rebless) = @_;
 
     $command = ucfirst(lc($command));
-    require "SVK/Command/$command.pm";
+    require "SVK/Command/$command.pm" unless $command =~ m/::/;
 
     my $cmd = (
         $is_rebless ? bless($self, "SVK::Command::$command")
@@ -783,12 +807,32 @@ sub find_checkout_anchor {
 }
 
 sub prompt_depotpath {
-    my ($self, $action) = @_;
+    my ($self, $action, $default) = @_;
 
-    my $path = get_prompt(loc(
-        "Enter a depot path to %1 into (under // if no leading '/'): ",
-        loc($action),
-    ));
+    my $path;
+
+    if (defined $default and $default =~ m{(^/[^/]*/)}) {
+        my $depot = $1;
+        my $prompt = loc("
+Next, please choose another depot path to serve as your private branch,
+usually something like %1your_project/.  SVK will copy the mirrored path
+into this new path, so you can commit into it without affecting the remote
+repository.  Enter a suitable name below to place it under %1; you may also
+press enter and accept the default.
+
+", $depot);
+        $path = get_prompt($prompt . loc(
+            "Enter a depot path to %1 into: [%2] ",
+            loc($action), $default
+        ));
+        $path = $default unless length $path;
+    }
+    else {
+        $path = get_prompt(loc(
+            "Enter a depot path to %1 into (under // if no leading '/'): ",
+            loc($action),
+        ));
+    }
 
     $path =~ s{^//+}{};
     $path =~ s{//+}{/};
@@ -812,7 +856,7 @@ Chia-liang Kao E<lt>clkao@clkao.orgE<gt>
 
 =head1 COPYRIGHT
 
-Copyright 2003-2004 by Chia-liang Kao E<lt>clkao@clkao.orgE<gt>.
+Copyright 2003-2005 by Chia-liang Kao E<lt>clkao@clkao.orgE<gt>.
 
 This program is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
