@@ -19,8 +19,6 @@ sub parse_arg {
     return $self->arg_co_maybe (@arg);
 }
 
-sub lock { $_[0]->lock_none }
-
 sub run {
     my ($self, $target) = @_;
 
@@ -85,20 +83,29 @@ SVK::Command::Annotate - Display per-line revision and author info
 
 =head1 SYNOPSIS
 
- annotate FILE
- annotate DEPOTPATH[@rev]
+ annotate [PATH][@REV]
+ annotate [-r REV] [PATH]
+ annotate DEPOTPATH[@REV]
+ annotate [-r REV] DEPOTPATH
 
 =head1 OPTIONS
 
+ -r [--revision] REV    : annotate up to revision
  -x [--cross]           : track revisions copied from elsewhere
 
+=head1 NOTES
+
+Note that -r REV file will do annotation up to REV,
+while file@REV will do annotation up to REV,
+plus the checkout copy differences.
+
 =head1 AUTHORS
 
 Chia-liang Kao E<lt>clkao@clkao.orgE<gt>
 
 =head1 COPYRIGHT
 
-Copyright 2003-2004 by Chia-liang Kao E<lt>clkao@clkao.orgE<gt>.
+Copyright 2003-2005 by Chia-liang Kao E<lt>clkao@clkao.orgE<gt>.
 
 This program is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
