@@ -28,7 +28,13 @@ sub parse_arg {
         # (otherwise it hurts when user types //deep/directory/name)
         $self->{parent} = 1;
 
-        my $path = $self->prompt_depotpath("copy");
+        # -- make a sane default here for mirroring --
+        my $default = undef;
+        if (@src == 1 and $src[0]->path =~ m{/mirror/([^/]+)$}) {
+            $default = "/" . $src[0]->depotname . "/$1";
+        }
+
+        my $path = $self->prompt_depotpath("copy", $default);
 
         if ($dst =~ /^\.?$/) {
             $self->{_checkout_path} = (splitdir($path))[-1];
@@ -45,7 +51,7 @@ sub parse_arg {
 
 sub lock {
     my $self = shift;
-    $_[-1]->{copath} ? $self->lock_target ($_[-1]) : $self->lock_none;
+    $self->lock_target ($_[-1]);
 }
 
 sub handle_co_item {
@@ -103,6 +109,7 @@ sub _unmodified {
 	    ( cb_flush => sub {
 		  die loc ("%1 is modified.\n", $target->copath ($_[0]));
 	      })),
+	  # need tests: only useful for move killing the src with unknown entries
 	  cb_unknown => sub {
 	      die loc ("%1 is missing.\n", $target->copath ($_[0]))});
 }
@@ -198,7 +205,7 @@ Chia-liang Kao E<lt>clkao@clkao.orgE<gt>
 
 =head1 COPYRIGHT
 
-Copyright 2003-2004 by Chia-liang Kao E<lt>clkao@clkao.orgE<gt>.
+Copyright 2003-2005 by Chia-liang Kao E<lt>clkao@clkao.orgE<gt>.
 
 This program is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
