@@ -5,7 +5,7 @@ our $VERSION = $SVK::VERSION;
 use base qw( SVK::Command );
 use SVK::XD;
 use SVK::I18N;
-use SVK::Util qw( get_buffer_from_editor get_prompt dirname abs_path move_path make_path $SEP );
+use SVK::Util qw( get_buffer_from_editor abs_path move_path );
 use YAML;
 use File::Path;
 
@@ -19,9 +19,12 @@ sub options {
 sub parse_arg {
     my ($self, @arg) = @_;
 
-    $self->{add} = 1 if @arg >= 2 and !$self->{relocate};
+    ++$self->{hold_giant};
+    $self->rebless ('depotmap::add')->{add} = 1 if @arg >= 2 and !$self->{relocate};
+
+    return undef
+	unless $self->{add} or $self->{detach} or $self->{relocate};
 
-    if ($self->{add} or $self->{detach} or $self->{relocate}) {
         @arg or die loc("Need to specify a depot name");
 
         my $depot = shift(@arg);
@@ -42,36 +45,10 @@ sub parse_arg {
         }
 
         return ($depot, @arg);
-    }
-    else {
-        return undef;
-    }
 }
 
 sub run {
     my ($self) = @_;
-
-    # Dispatch to one of the four methods
-    foreach my $op (qw( list add detach relocate )) {
-        $self->{$op} or next;
-        goto &{ $self->can("_do_$op") };
-    }
-
-    return $self->_do_edit();
-}
-
-sub _do_list {
-    my ($self) = @_;
-    my $map = $self->{xd}{depotmap};
-    my $fmt = "%-20s\t%-s\n";
-    printf $fmt, loc('Depot'), loc('Path');
-    print '=' x 60, "\n";
-    printf $fmt, "/$_/", $map->{$_} for sort keys %$map;
-    return;
-}
-
-sub _do_edit {
-    my ($self) = @_;
     my $sep = '===edit the above depot map===';
     my $map = YAML::Dump ($self->{xd}{depotmap});
     my $new;
@@ -86,11 +63,15 @@ sub _do_edit {
         print loc("New depot map saved.\n");
         $self->{xd}{depotmap} = $new;
     }
-    $self->create_depots;
+    $self->{xd}->create_depots;
     return;
 }
 
-sub _do_add {
+package SVK::Command::Depotmap::add;
+use base qw(SVK::Command::Depotmap);
+use SVK::I18N;
+
+sub run {
     my ($self, $depot, $path) = @_;
 
     die loc("Depot '%1' already exists; use 'svk depotmap --detach' to remove it first.\n", $depot)
@@ -99,10 +80,14 @@ sub _do_add {
     $self->{xd}{depotmap}{$depot} = $path;
 
     print loc("New depot map saved.\n");
-    $self->create_depots;
+    $self->{xd}->create_depots;
 }
 
-sub _do_relocate {
+package SVK::Command::Depotmap::relocate;
+use base qw(SVK::Command::Depotmap);
+use SVK::I18N;
+
+sub run {
     my ($self, $depot, $path) = @_;
 
     die loc("Depot '%1' does not exist in the depot map.\n", $depot)
@@ -111,12 +96,15 @@ sub _do_relocate {
     $self->{xd}{depotmap}{$depot} = $path;
 
     print loc("Depot '%1' relocated to '%2'.\n", $depot, $path);
-    $self->create_depots;
+    $self->{xd}->create_depots;
 }
 
-sub _do_detach {
-    my ($self, $depot) = @_;
+package SVK::Command::Depotmap::detach;
+use base qw(SVK::Command::Depotmap);
+use SVK::I18N;
 
+sub run {
+    my ($self, $depot) = @_;
     delete $self->{xd}{depotmap}{$depot}
         or die loc("Depot '%1' does not exist in the depot map.\n", $depot);
 
@@ -124,26 +112,19 @@ sub _do_detach {
     return;
 }
 
-sub create_depots {
+package SVK::Command::Depotmap::list;
+use base qw(SVK::Command::Depotmap);
+use SVK::I18N;
+
+sub parse_arg { undef }
+
+sub run {
     my ($self) = @_;
-    for my $path (values %{$self->{xd}{depotmap}}) {
-        $path =~ s{[$SEP/]+$}{}go;
-
-	next if -d $path;
-	my $ans = get_prompt(
-	    loc("Repository %1 does not exist, create? (y/n)", $path),
-	    qr/^[yn]/i,
-	);
-	next if $ans =~ /^n/i;
-
-        make_path(dirname($path));
-
-        $ENV{SVNFSTYPE} ||= (($SVN::Core::VERSION =~ /^1\.0/) ? 'bdb' : 'fsfs');
-	SVN::Repos::create($path, undef, undef, undef,
-			   {'fs-type' => $ENV{SVNFSTYPE},
-			    'bdb-txn-nosync' => '1',
-			    'bdb-log-autoremove' => '1'});
-    }
+    my $map = $self->{xd}{depotmap};
+    my $fmt = "%-20s\t%-s\n";
+    printf $fmt, loc('Depot'), loc('Path');
+    print '=' x 60, "\n";
+    printf $fmt, "/$_/", $map->{$_} for sort keys %$map;
     return;
 }
 
@@ -158,11 +139,11 @@ SVK::Command::Depotmap - Create or edit the depot mapping configuration
 =head1 SYNOPSIS
 
  depotmap [OPTIONS]
- depotmap DEPOT /path/to/repository
+ depotmap DEPOTNAME /path/to/repository
 
  depotmap --list
- depotmap --detach [DEPOT|PATH]
- depotmap --relocate [DEPOT|PATH] PATH
+ depotmap --detach [DEPOTNAME | PATH]
+ depotmap --relocate [DEPOTNAME | PATH] PATH
 
 =head1 OPTIONS
 
@@ -184,13 +165,16 @@ The depotname may then be used as part of a DEPOTPATH:
 
  /depotname/path/inside/repos
 
+Depot creation respects $ENV{SVNFSTYPE}, which is default to fsfs for
+svn 1.1 or later, and bdb for svn 1.0.x.
+
 =head1 AUTHORS
 
 Chia-liang Kao E<lt>clkao@clkao.orgE<gt>
 
 =head1 COPYRIGHT
 
-Copyright 2003-2004 by Chia-liang Kao E<lt>clkao@clkao.orgE<gt>.
+Copyright 2003-2005 by Chia-liang Kao E<lt>clkao@clkao.orgE<gt>.
 
 This program is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
