@@ -20,8 +20,6 @@ sub parse_arg {
     return map {$self->arg_co_maybe ($_)} @arg;
 }
 
-sub lock { $_[0]->lock_none }
-
 # XXX: need to handle peg revisions, ie
 # -r N PATH@M means the node PATH@M at rev N
 sub run {
@@ -43,6 +41,9 @@ sub run {
 	    die loc("invalid arguments") if $target->{copath};
 	    # prevent oldroot being xdroot below
 	    $r1 ||= $yrev;
+	    # diff DEPOTPATH COPATH require DEPOTPATH to exist
+	    die loc("path %1 does not exist.\n", $target->{report})
+		if $fs->revision_root ($r1)->check_path ($target->{path}) == $SVN::Node::none;
 	}
     }
     else {
@@ -97,8 +98,9 @@ sub run {
 	  oldtarget => $target, oldroot => $oldroot,
 	);
 
+    my $kind = $oldroot->check_path ($target->{path});
     if ($target2->{copath}) {
-	if ($oldroot->check_path ($target2->{path}) != $SVN::Node::dir) {
+	if ($kind != $SVN::Node::dir) {
 	    my $tgt;
 	    ($target2->{path}, $tgt) = get_anchor (1, $target2->{path});
 	    ($target->{path}, $target2->{copath}) =
@@ -106,7 +108,6 @@ sub run {
 	    $target2->{targets} = [$tgt];
 	    ($report) = get_anchor (0, $report) if defined $report;
 	}
-
 	$editor->{report} = $report;
 	$self->{xd}->checkout_delta
 	    ( %$target2,
@@ -119,7 +120,10 @@ sub run {
     }
     else {
 	my $tgt = '';
-	if ($oldroot->check_path ($target2->{path}) != $SVN::Node::dir) {
+	die loc("path %1 does not exist.\n", $target->{report})
+	    if $kind == $SVN::Node::none;
+
+	if ($kind != $SVN::Node::dir) {
 	    ($target->{path}, $tgt) =
 		get_anchor (1, $target->{path});
 	    ($report) = get_anchor (0, $report) if defined $report;
@@ -164,7 +168,7 @@ Chia-liang Kao E<lt>clkao@clkao.orgE<gt>
 
 =head1 COPYRIGHT
 
-Copyright 2003-2004 by Chia-liang Kao E<lt>clkao@clkao.orgE<gt>.
+Copyright 2003-2005 by Chia-liang Kao E<lt>clkao@clkao.orgE<gt>.
 
 This program is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
