@@ -18,7 +18,7 @@ sub options {
     ('depot=s' => 'depot');
 }
 
-sub lock { $_[0]->lock_none }
+# XXX: lock for apply?
 
 sub parse_arg {
     my ($self, $cmd, @arg) = @_;
@@ -164,7 +164,7 @@ Chia-liang Kao E<lt>clkao@clkao.orgE<gt>
 
 =head1 COPYRIGHT
 
-Copyright 2003-2004 by Chia-liang Kao E<lt>clkao@clkao.orgE<gt>.
+Copyright 2003-2005 by Chia-liang Kao E<lt>clkao@clkao.orgE<gt>.
 
 This program is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
