@@ -14,8 +14,7 @@ sub parse_arg {
 
 sub lock {
     my $self = shift;
-    $_->{copath} ? $self->lock_target ($_) : $self->lock_none
-	for (@_[1..$#_]);
+    $self->lock_target (@_[1..$#_]);
 }
 
 sub run {
@@ -34,7 +33,7 @@ SVK::Command::Propdel - Delete a property on files or dirs
 
 =head1 SYNOPSIS
 
- propdel PROPNAME [PATH|DEPOTPATH...]
+ propdel PROPNAME [DEPOTPATH | PATH...]
 
 =head1 OPTIONS
 
@@ -49,7 +48,7 @@ Chia-liang Kao E<lt>clkao@clkao.orgE<gt>
 
 =head1 COPYRIGHT
 
-Copyright 2003-2004 by Chia-liang Kao E<lt>clkao@clkao.orgE<gt>.
+Copyright 2003-2005 by Chia-liang Kao E<lt>clkao@clkao.orgE<gt>.
 
 This program is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
