@@ -16,7 +16,7 @@ sub parse_arg {
 
 sub lock {
     my $self = shift;
-    $_[1]->{copath} ? $self->lock_target ($_[1]) : $self->lock_none;
+    $self->lock_target ($_[1]);
 }
 
 sub run {
@@ -65,7 +65,7 @@ Chia-liang Kao E<lt>clkao@clkao.orgE<gt>
 
 =head1 COPYRIGHT
 
-Copyright 2003-2004 by Chia-liang Kao E<lt>clkao@clkao.orgE<gt>.
+Copyright 2003-2005 by Chia-liang Kao E<lt>clkao@clkao.orgE<gt>.
 
 This program is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
