@@ -15,7 +15,11 @@ sub parse_arg {
 
     @arg = ('') if $#arg < 0;
 
-    if ( @arg == 1 and !$self->arg_co_maybe($arg[0])->{'copath'}) {
+    if ($self->{all}) {
+        my $checkout = $self->{xd}{checkout}{hash};
+        @arg = sort grep $checkout->{$_}{depotpath}, keys %$checkout;
+    } 
+    elsif ( @arg == 1 and !$self->arg_co_maybe($arg[0])->{'copath'}) {
         # If the last argument is a depot path, rather than a copath
         # then we should do a merge to the local depot, rather than 
         # an update to the path
@@ -29,11 +33,6 @@ sub parse_arg {
         )->parse_arg (@arg);
     }
 
-    if ($self->{all}) {
-        my $checkout = $self->{xd}{checkout}{hash};
-        @arg = sort grep $checkout->{$_}{depotpath}, keys %$checkout;
-    } 
-
     $self->{lump} = 1; # -- XXX -- will break otherwise -- XXX ---
 
     $self->{sync}++;
@@ -74,7 +73,7 @@ Chia-liang Kao E<lt>clkao@clkao.orgE<gt>
 
 =head1 COPYRIGHT
 
-Copyright 2003-2004 by Chia-liang Kao E<lt>clkao@clkao.orgE<gt>.
+Copyright 2003-2005 by Chia-liang Kao E<lt>clkao@clkao.orgE<gt>.
 
 This program is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
