@@ -3,7 +3,8 @@ use strict;
 our $VERSION = $SVK::VERSION;
 use SVK::XD;
 use SVK::I18N;
-use SVK::Util qw( get_anchor catfile abs2rel HAS_SVN_MIRROR IS_WIN32 );
+use SVK::Util qw( get_anchor catfile abs2rel HAS_SVN_MIRROR IS_WIN32
+		  find_prev_copy );
 use SVK::Target::Universal;
 use Clone;
 
@@ -231,13 +232,16 @@ sub depotname {
 sub copy_ancestors {
     my $self = shift;
     my $fs = $self->{repos}->fs;
-    my $t = $self->new;
-    warn "==> ".$t->path;
+    my $t = $self->new->as_depotpath;
+    my @result;
     while (my ($copyto, $copyfrom_rev, $copyfrom_path) = $t->nearest_copy) {
 	$t->{path} = $copyfrom_path;
 	$t->{revision} = $copyfrom_rev;
-	warn "==> . ".$t->path." @ $t->{revision}";
+	push @result, [@{$t}{qw(path revision)}];
+	# if the original target is anchorified, do so too.
+	delete $t->{targets}, $t->anchorify if exists $self->{targets}[0];
     }
+    return @result;
 }
 
 use List::Util qw(min);
@@ -249,37 +253,46 @@ sub nearest_copy {
     if (ref ($root) eq __PACKAGE__) {
 	($root, $path) = ($root->root, $root->path);
     }
-    # normalize;
+    my $fs = $root->fs;
+    my $spool = SVN::Pool->new_default;
+    my $old_pool = SVN::Pool->new;
+    my $new_pool = SVN::Pool->new;
+
+    # normalize
     my $histself = $root->node_history ($path)->prev(0);
     my $rev = ($histself->location)[1];
-
-    my $fs = $root->fs;
+    my $lastcopy;
     while (1) {
 	# Find history_prev revision, if the path is different, bingo.
-	my ($hppath, $hprev);
-	if (my $hist = $histself->prev(1)) {
-	    ($hppath, $hprev) = $hist->location;
-	    if ($hppath ne $path) {
-		return ($rev, $hprev, $hppath);
-	    }
-	}
+	my $hist = $histself->prev(1, $new_pool) or last; # no more history
+	my ($hppath, $hprev) = $hist->location;
+	return ($rev, $hprev, $hppath) if $hppath ne $path; # got it
+	$histself = $hist;
 
 	# Find nearest copy of the current revision (up to but *not*
 	# including the revision itself). If the copy contains us, bingo.
-	my ($prev, $copy) = _find_prev_copy ($fs, $rev-1);
-	if ($copy && (my ($fromrev, $frompath) = _copies_contain_path ($copy, $path))) {
+	my ($prev, $copy) = find_prev_copy ($fs,
+					    $lastcopy && $lastcopy == $rev
+					    ? $rev-1
+					    : $rev);
+	last unless $prev; # no more copies
+	if (my ($fromrev, $frompath) = _copies_contain_path ($copy, $path)) {
+	    # there were copy, but the descendent might not exist there
+	    last unless $fs->revision_root ($fromrev)->check_path ($frompath);
 	    return ($prev, $fromrev, $frompath);
 	}
-	# Continue testing on min (history_prev, prev_copy), provided
-	# it's still a related to the current node.
-	$rev = min (grep defined, $prev, $hprev) or last;
-
-	# Reset the hprev root to this earlier revision to avoid infinite looping
-	$root = $fs->revision_root ($rev);
-	if ($root->check_path ($path) == $SVN::Node::none) {
-	    last;
+	elsif ($prev < $hprev) {
+	    # Reset the hprev root to this earlier revision to avoid infinite looping
+	    last unless $fs->revision_root ($prev)->check_path ($path);
+	    $root = $fs->revision_root ($prev, $new_pool);
+	    $histself = $root->node_history ($path)->prev(0, $new_pool);
 	}
-	$histself = $root->node_history ($path)->prev(0);
+	$lastcopy = $prev;
+	# Continue testing on min (history_prev, prev_copy)
+	$rev = min ($prev, $hprev);
+        $old_pool->clear;
+	$spool->clear;
+        ($old_pool, $new_pool) = ($new_pool, $old_pool);
     }
     return;
 }
@@ -293,39 +306,6 @@ sub _copies_contain_path {
     return ($copy->{$match}[0], $path);
 }
 
-sub _copies_in_rev {
-    my ($fs, $rev) = @_;
-    my $copies;
-    my $root = $fs->revision_root ($rev);
-    my $changed = $root->paths_changed;
-    for (keys %$changed) {
-	next if $changed->{$_}->change_kind == $SVN::Fs::PathChange::delete;
-	my ($copyfrom_rev, $copyfrom_path) = $root->copied_from ($_);
-	$copies->{$_} = [$copyfrom_rev, $copyfrom_path]
-	    if defined $copyfrom_path;
-    }
-    return $copies;
-}
-
-sub _find_prev_copy {
-    my ($fs, $endrev) = @_;
-    my $pool = SVN::Pool->new_default;
-    my $rev = $endrev;
-    while ($rev > 0) {
-	$pool->clear;
-	if (my $cache = $fs->revision_prop ($rev, 'svk:copy_cache_prev')) {
-	    $rev = $cache;
-	}
-	if (my $copy = _copies_in_rev ($fs, $rev)) {
-	    $fs->change_rev_prop ($_, 'svk:copy_cache_prev', $rev)
-		for $rev..$endrev;
-	    return ($rev, $copy);
-	}
-	--$rev;
-    }
-    return undef;
-}
-
 =head2 related_to
 
 Check if C<$self> is related to another target.
@@ -396,7 +376,7 @@ Chia-liang Kao E<lt>clkao@clkao.orgE<gt>
 
 =head1 COPYRIGHT
 
-Copyright 2003-2004 by Chia-liang Kao E<lt>clkao@clkao.orgE<gt>.
+Copyright 2003-2005 by Chia-liang Kao E<lt>clkao@clkao.orgE<gt>.
 
 This program is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
