@@ -12,7 +12,7 @@ use SVK::Editor::XD;
 use SVK::I18N;
 use SVK::Util qw( get_anchor abs_path abs2rel splitdir catdir splitpath $SEP
 		  HAS_SYMLINK is_symlink is_executable mimetype mimetype_is_text
-		  md5_fh  traverse_history );
+		  md5_fh get_prompt traverse_history make_path dirname );
 use Data::Hierarchy 0.21;
 use File::Spec;
 use File::Find;
@@ -133,8 +133,39 @@ sub load {
     $info ||= { depotmap => {'' => catdir($self->{svkpath}, 'local') },
 	        checkout => Data::Hierarchy->new( sep => $SEP ) };
     $self->{$_} = $info->{$_} for keys %$info;
+
+    $self->create_depots('');
 }
 
+=item store
+
+=cut
+
+sub create_depots {
+    my $self = shift;
+    my $depotmap = $self->{depotmap};
+    for my $path (@{$depotmap}{sort (@_ ? @_ : keys %$depotmap)}) {
+        $path =~ s{[$SEP/]+$}{}go;
+
+	next if -d $path;
+	my $ans = get_prompt(
+	    loc("Repository %1 does not exist, create? (y/n)", $path),
+	    qr/^[yn]/i,
+	);
+	next if $ans =~ /^n/i;
+
+        make_path(dirname($path));
+
+        $ENV{SVNFSTYPE} ||= (($SVN::Core::VERSION =~ /^1\.0/) ? 'bdb' : 'fsfs');
+	SVN::Repos::create($path, undef, undef, undef,
+			   {'fs-type' => $ENV{SVNFSTYPE},
+			    'bdb-txn-nosync' => '1',
+			    'bdb-log-autoremove' => '1'});
+    }
+    return;
+}
+
+
 =item store
 
 Serialize C<$self> to the statefile. If giant lock is still ours,
@@ -146,8 +177,14 @@ giant is unlocked.
 
 sub _store_self {
     my ($self, $hash) = @_;
-    DumpFile ($self->{statefile},
+    local $SIG{INT};
+    my $file = $self->{statefile};
+    my $tmpfile = $file."-$$";
+    DumpFile ($tmpfile,
 	      { map { $_ => $hash->{$_}} qw/checkout depotmap/ });
+    unlink ("$file~");
+    rename ($file => "$file~");
+    rename ($tmpfile => $file);
 }
 
 sub store {
@@ -281,7 +318,7 @@ sub find_repos {
     my ($depot, $path) = $depotpath =~ m|^/([^/]*)(/.*?)/?$|
 	or die loc("%1 is not a depot path.\n", $depotpath);
 
-    my $repospath = $self->{depotmap}{$depot} or die loc("no such depot: %1", $depot);
+    my $repospath = $self->{depotmap}{$depot} or die loc("No such depot: %1.\n", $depot);
 
     return ($repospath, $path, $open && _open_repos ($repospath));
 }
@@ -434,6 +471,17 @@ sub xd_storage_cb {
 			       $self->{checkout}->store ($_, {'.conflict' => 1})
 				   unless $arg{check_only};
 			   },
+	  cb_prop_merged => sub { return if $arg{check_only};
+				  $_ = shift; $arg{get_copath} ($_);
+				  my $name = shift;
+				  my $entry = $self->{checkout}->get ($_);
+				  my $prop = $entry->{'.newprop'};
+				  delete $prop->{$name};
+				  $self->{checkout}->store ($_, {'.newprop' => $prop,
+								 keys %$prop ? () :
+								 ('.schedule' => undef)}
+								);
+			      },
 	  cb_localmod => sub { my ($path, $checksum) = @_;
 			       my $copath = $path;
 			       # XXX: make use of the signature here too
@@ -586,7 +634,7 @@ sub do_delete {
 					push @deleted, $copath;
 				    }
 				    elsif (-f $copath) {
-					die loc("%1 is scheduled, use 'svk revert'", $report);
+					die loc("%1 is scheduled, use 'svk revert'.\n", $report);
 				    }
 				})),
 			    cb_unknown => sub {
@@ -598,7 +646,7 @@ sub do_delete {
     my @paths = grep {is_symlink($_) || -e $_} (exists $arg{targets}[0] ?
 			      map { SVK::Target->copath ($arg{copath}, $_) } @{$arg{targets}}
 			      : $arg{copath});
-    my $ignore = ignore ();
+    my $ignore = $self->ignore ();
     find(sub {
 	     return if m/$ignore/;
 	     my $cpath = catdir($File::Find::dir, $_);
@@ -748,9 +796,14 @@ and treat all copied descendents as added too.
 my %ignore_cache;
 
 sub ignore {
+    my $self = shift;
     no warnings;
-    my @ignore = qw/*.o #*# .#* *.lo *.la .*.rej *.rej .*~ *~ .DS_Store
-		    svk-commit*.tmp/;
+    my $ignore = $self->{svnconfig} ?
+	           $self->{svnconfig}{config}->
+		   get ('miscellany', 'global-ignores', '') : '';
+    my @ignore = split / /,
+	($ignore || "*.o *.lo *.la #*# .*.rej *.rej .*~ *~ .#* .DS_Store");
+    push @ignore, 'svk-commit*.tmp';
 
     return join('|', map {$ignore_cache{$_} ||= compile_shellish $_} (@ignore, @_));
 }
@@ -775,7 +828,7 @@ sub _delta_content {
 
 sub _unknown_verbose {
     my ($self, %arg) = @_;
-    my $ignore = ignore;
+    my $ignore = $self->ignore;
     # The caller should have processed the entry already.
     my %seen = ($arg{copath} => 1);
     if ($arg{targets}) {
@@ -1069,10 +1122,11 @@ sub _delta_dir {
 	$signature->flush;
 	undef $signature;
     }
-    my $ignore = ignore (split ("\n", $fullprops->{'svn:ignore'} || ''));
+    my $ignore = $self->ignore (split ("\n", $fullprops->{'svn:ignore'} || ''));
 
     my @direntries;
-    unless (defined $targets && !keys %$targets) {
+    # if we are at somewhere arg{copath} not exist, $arg{type} is empty
+    if ($arg{type} && !(defined $targets && !keys %$targets)) {
 	opendir my ($dir), $arg{copath} or die "$arg{copath}: $!";
 	@direntries = sort grep { !m/^\.+$/ && !exists $entries->{$_} } readdir ($dir);
     }
@@ -1098,7 +1152,11 @@ sub _delta_dir {
 	if (!defined $targets) {
 	    next if (!$add || $arg{auto_add}) && $entry =~ m/$ignore/ ;
 	}
-	unless ($add) {
+	if ($ccinfo->{'.conflict'}) {
+	    $arg{cb_conflict}->($arg{editor}, $newpaths{entry}, $arg{baton})
+		if $arg{cb_conflict};
+	}
+	unless ($add || $ccinfo->{'.conflict'}) {
 	    if ($arg{cb_unknown}) {
 		$arg{cb_unknown}->($newpaths{entry}, $newpaths{copath});
 		$self->_unknown_verbose (%arg, %newpaths)
@@ -1557,7 +1615,7 @@ Chia-liang Kao E<lt>clkao@clkao.orgE<gt>
 
 =head1 COPYRIGHT
 
-Copyright 2003-2004 by Chia-liang Kao E<lt>clkao@clkao.orgE<gt>.
+Copyright 2003-2005 by Chia-liang Kao E<lt>clkao@clkao.orgE<gt>.
 
 This program is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
